/*
 *	Sherlock Filter Engine -- Lexer
 *
 *	(c) 1999--2000 Martin Mares <mj@ucw.cz>
 *	(c) 2001 Robert Spalek <robert@ucw.cz>
 */

%{
#undef	REJECT			/* internal unused flex macro */

#include "lib/lib.h"
#include "filter/filter.h"
#include "filter/parse.tab.h"

#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

#define err(x) filter_err(x)

#define	YY_NO_UNPUT
#define YY_DECL	int yylex(void)

#define KW(x) { #x, x }
static struct keyword {
	byte *name;
	int value;
} keyword_list[] =
{
	KW(ACCEPT),	/* commands */
	KW(REJECT),
	{ "ERROR", ERROR1 },
	{ "WARNING", WARNING1 },
	{ "LOG", LOG1 },
	{ "DEBUG", DEBUG1 },

	KW(IF),		/* statements */
	KW(ELSE),
	KW(UNDEF),
	KW(ELIF),
	KW(SWITCH),
	KW(CASE),

	KW(TRUE),	/* expressions */
	KW(FALSE),
	KW(UNDEFINED),
	KW(DEFINED),
	KW(ATTR),
	KW(ADD),
	KW(DELETE),
	KW(CONF),
	KW(INT),
	KW(STRING),
	KW(REGEXP),

	{ NULL, -1 }
};
#undef KW

static int lino;
static byte *filter_name;

%}

%option noyywrap

%x COMMENT CCOMM

ALPHA [a-zA-Z_]
DIGIT [0-9]
XIGIT [0-9a-fA-F]
ALNUM [a-zA-Z_0-9]
WHITE [ \t]

%%

0x{XIGIT}+ {
	char *e;
	long int l;
	errno = 0;
	l = strtoul(yytext+2, &e, 16);
	if (e && *e || errno == ERANGE || (long int)(int) l != l)
		err("Number out of range");
	yylval.i = l;
	return NUM;
}

0{DIGIT}+ {
	char *e;
	long int l;
	errno = 0;
	l = strtoul(yytext+1, &e, 8);
	if (e && *e || errno == ERANGE || (long int)(int) l != l)
		err("Number out of range");
	yylval.i = l;
	return NUM;
}

-?{DIGIT}+ {
	char *e;
	long int l;
	errno = 0;
	l = strtoul(yytext, &e, 10);
	if (e && *e || errno == ERANGE || (long int)(int) l != l)
		err("Number out of range");
	yylval.i = l;
	return NUM;
}

{ALPHA}{ALNUM}* {
	struct keyword *k;
	struct filter_lex_name *name;
	for (k = keyword_list; k->name; k++)
		if (!strcasecmp(k->name, yytext))
			return k->value;
	for (name = &filter_current->name_head; name->next; name = name->next)
		if (!strcasecmp(name->next->name, yytext))
		{
			yylval.s = name->next->name;
			return IDENT;
		}
	name->next = filter_alloc(sizeof(struct filter_lex_name) + yyleng+1);
	name = name->next;
	name->next = NULL;
	strncpy(name->name, yytext, yyleng);
	name->name[yyleng] = 0;
	yylval.s = name->name;
	return IDENT;
}

"<"	return LT;
">"	return GT;
"<="	return LE;
">="	return GE;
"=="	return EQ;
"!="	return NE;
"=~"	return EREG;
"!~"	return NREG;
"=*"	return EPAT;
"!*"	return NPAT;

"<<"	return LTC;
">>"	return GTC;
"<=="	return LEC;
">=="	return GEC;
"==="	return EQC;
"!=="	return NEC;
"=~~"	return EREGC;
"!~~"	return NREGC;
"=**"	return EPATC;
"!**"	return NPATC;

"&&"	return AND;
"||"	return OR;

"~"	return PATTERN;
"~~"	return IPATTERN;

[.,:;(){}\[\]+\-*/%^&|!=<>] {
	return yytext[0];
}

["][^"\n]*["] {
	yytext[yyleng-1] = 0;
	yylval.s = filter_stralloc(yytext+1);
	return STRING;
}

["][^"\n]*\n	err("Unterminated string");

<INITIAL,COMMENT><<EOF>>	return END;

{WHITE}+

\n	lino++;

#	BEGIN(COMMENT);

\/\*	BEGIN(CCOMM);

.	err("Unknown character");

<COMMENT>\n {
	lino++;
	BEGIN(INITIAL);
}

<COMMENT>.

<CCOMM>\*\/	BEGIN(INITIAL);
<CCOMM>\n	lino++;
<CCOMM>\/\*	err("Comment nesting not supported");
<CCOMM><<EOF>>	err("Unterminated comment");
<CCOMM>.

%%

void
filter_lex_init(byte *name)
{
	lino = 1;
	filter_name = name;
	yyin = fopen(name, "r");
	if (!yyin)
		die("Unable to open %s: %m", name);
}

void
filter_lex_cleanup(void)
{
	fclose(yyin);
}

void
filter_err(byte *msg)
{
	die("%s, line %d: %s", filter_name, lino, msg);
}
