/*
 *	Sherlock Gatherer: Protocol Multiplexer
 *
 *	(c) 2001--2002 Martin Mares <mj@ucw.cz>
 */

#include "lib/lib.h"
#include "lib/ipaccess.h"
#include "gather/gather.h"

#include <stdlib.h>
#include <netdb.h>
#include <netinet/in.h>

typedef void (*downloader_t)(void);

static downloader_t downloaders[URL_PROTO_MAX] = {
  [URL_PROTO_HTTP] =	http_download,
  [URL_PROTO_FILE] =	file_download,
};

void
download(void)
{
  if (!downloaders[gthis->url_s.protoid])
    gerror(2100, "Unknown protocol");
  gather_filter();
  if (gthis->refreshing)
    {
      byte *last_dwn = obj_find_aval(gthis->refreshing, 'D');
      byte *last_mod = obj_find_aval(gthis->refreshing, 'L');
      if (last_dwn && gthis->download_time - (sh_time_t)atol(last_dwn) <= max_refresh_age)
	{
	  if (last_mod && min_ims_delay >= 0)
	    gthis->if_modified_since_time = atol(last_mod) + min_ims_delay;
	  gthis->if_different_etag = obj_find_aval(gthis->refreshing, 'g');
	}
    }
  gthis->temp = fbmem_create(16384);
  downloaders[gthis->url_s.protoid]();
  bflush(gthis->temp);
  gthis->contents = gthis->temp;
  gthis->temp = NULL;
}

void
create_key(void)
{
  u32 key;
  byte buf[32];

  switch (gthis->url_s.protoid)
    {
    case URL_PROTO_HTTP:
      key = ntohl(resolve_host_name(gthis->url_s.host));
      break;
    default:
      key = 0x7f010000 + random_max(1024);
    }
  sprintf(buf, "%08x", key);
  obj_set_attr(gthis->aa, 'k', buf);
  gerror(2, "Resolved");
}

u32
resolve_host_name(byte *name)
{
  u32 addr;
  struct hostent *h;

  h = gethostbyname(name);
  if (!h)
    {
      switch (h_errno)
	{
	case HOST_NOT_FOUND:
	case NO_ADDRESS:
	  gerror(2103, "Host doesn't exist");
	case TRY_AGAIN:
	  gerror(1104, "DNS timeout");
	default:
	  gerror(2105, "Unrecoverable DNS error");
	}
    }
  memcpy(&addr, h->h_addr, sizeof(addr));
  if (!ipaccess_check(gaccess_list, ntohl(addr)))
    gerror(2134, "Forbidden IP address %08x", ntohl(addr));
  return addr;
}
