/*
 *	Sherlock Search Engine -- Lexical Analysis of Queries
 *
 *	(c) 1997--2002 Martin Mares <mj@ucw.cz>
 */

#include "lib/lib.h"
#include "lib/chartype.h"
#include "search/sherlockd.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "search/parse.tab.h"

static byte *pos;

void
lex_init(byte *buf)
{
  pos = buf;
}

struct keywd {
  char *keyword;
  uns class, val;
};

static struct keywd keywords[] = {
  { "ANY",			ANY,		0 },
  { "AND",			AND,		0 },
  { "OR",			OR,		0 },
  { "NOT",			NOT,		0 },
  { "DEBUG",			K_DEBUG,	0 },
  { "CONTROL",			CONTROL,	0 },
  { "ACCENTS",			ACCENTS,	0 },
  { "LIST",			LIST,		0 },
  { "SHOW",			SHOW,		0 },
  { "CONTEXT",			CONTEXT,	0 },
  { "TITLELEN",			TITLELEN,	0 },
  { "DB",			DB,		0 },
  { "SITE",			SITE,		0 },
  { "URLS",			URLS,		0 },
  { "SITEMAX",			SITEMAX,	0 },
  { "PARTIAL",			PARTIAL,	0 },
  { "APPROX",			APPROX,		0 },
  { "MAYBE",			MAYBE,		0 },
  { "STATS",			STATS,		0 },
  { "SORTBY",			SORTBY,		0 },
#define T(x,y) { #x, REF, y },
WORD_TYPE_NAMES
#undef T
#define T(x,y) { #x, REF, 0x40000000 | (y) },
STRING_TYPE_NAMES
#undef T
#define INT_ATTR(id,keywd,gf,pf) { #keywd, CUSTOM, OFFSETOF(struct query, id##_min) },
#define SMALL_SET_ATTR(id,keywd,gf,pf) { #keywd, CUSTOM, OFFSETOF(struct query, id##_set) },
  CUSTOM_ATTRS
#undef INT_ATTR
#undef SMALL_SET_ATTR
  { "AGE",			CUSTOM,		PARAM_AGE },
  /* SITE again, now with class CUSTOM for sake of lookup_custom_attr() */
  { "SITE",			CUSTOM,		PARAM_SITE },
};

int
yylex(void)
{
  while (Cspace(*pos))
    pos++;
  if (!*pos)
    return EOLN;
  if (Cdigit(*pos))
    {
      uns i = 0;
      while (Cdigit(*pos))
	{
	  if (i > 214748364)
	    err("Number too large");
	  i = 10*i + *pos++ - '0';
	  if (i >= 2147483648U)
	    err("Number too large");
	}
      yylval.i = i;
      return NUM;
    }
  if (Calpha(*pos))
    {
      byte *start = pos;
      byte x;
      uns k;

      while (Calpha(*pos))
	{
	  *pos = Cupcase(*pos);
	  pos++;
	}
      x = *pos;
      *pos = 0;
      for(k=0; k<ARRAY_SIZE(keywords); k++)
	if (!strcmp(start, keywords[k].keyword))
	  {
	    *pos = x;
	    yylval.i = keywords[k].val;
	    return keywords[k].class;
	  }
      err("Unknown keyword");
    }
  switch (*pos++)
    {
    case '"':
      {
	byte *z = pos;
	while (*pos && *pos != '"')
	  pos++;
	if (!*pos)
	  err("Misquoted string");
	*pos++ = 0;
	yylval.s = z;
	return STRING;
      }
    case '&':
      return AND;
    case '|':
      return OR;
    case '!':
      return NOT;
    case '.':
      if (*pos == '.')
	{
	  pos++;
	  return DOTDOT;
	}
      /* fall-thru */
    case ',':
    case '/':
    case '(':
    case ')':
    case '-':
    case '=':
    case '{':
    case '}':
      return pos[-1];
    case '<':
      if (*pos == '=')
	{
	  pos++;
	  return LE;
	}
      else if (*pos == '>')
	{
	  pos++;
	  return NE;
	}
      return '<';
    case '>':
      if (*pos == '=')
	{
	  pos++;
	  return GE;
	}
      return '>';
    case '#':
      {
	u32 i = 0;
	if (!Cxdigit(*pos))
	  err("Invalid site ID");
	while (Cxdigit(*pos))
	  {
	    if (i >= 0x10000000)
	      err("Site ID too large");
	    i = (i << 4) | Cxvalue(*pos);
	    pos++;
	  }
	yylval.i = i;
	return SITEID;
      }
    default:
      err("Unknown character");
    }
}

int
lookup_custom_attr(byte *name)
{
  uns k;

  for(k=0; k<ARRAY_SIZE(keywords); k++)
    if (keywords[k].class == CUSTOM && !strcasecmp(name, keywords[k].keyword))
      return keywords[k].val;
  return -1;
}
